import { __extends, __read, __assign } from 'tslib';
import React__default, { useContext } from 'react';
import { usePresence } from '../../../components/AnimatePresence/use-presence.js';
import { LayoutGroupContext } from '../../../context/LayoutGroupContext.js';
import { correctBorderRadius } from '../../../projection/styles/scale-border-radius.js';
import { correctBoxShadow } from '../../../projection/styles/scale-box-shadow.js';
import { addScaleCorrector } from '../../../projection/styles/scale-correction.js';
import { HTMLProjectionNode } from '../../../projection/node/HTMLProjectionNode.js';

var MeasureLayoutWithContext = /** @class */ (function (_super) {
    __extends(MeasureLayoutWithContext, _super);
    function MeasureLayoutWithContext() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * This only mounts projection nodes for components that
     * need measuring, we might want to do it for all components
     * in order to incorporate transforms
     */
    MeasureLayoutWithContext.prototype.componentDidMount = function () {
        var _a;
        var visualElement = this.props.visualElement;
        var projection = visualElement.projection;
        addScaleCorrector(defaultScaleCorrectors);
        var group = this.context.group;
        if (group)
            group.add(projection);
        (_a = this.props.visualElement.projection) === null || _a === void 0 ? void 0 : _a.root.didUpdate();
    };
    MeasureLayoutWithContext.prototype.getSnapshotBeforeUpdate = function (prevProps) {
        var _a;
        var _b = this.props, layoutDependency = _b.layoutDependency, visualElement = _b.visualElement, drag = _b.drag;
        if (drag ||
            prevProps.layoutDependency !== layoutDependency ||
            layoutDependency === undefined) {
            (_a = visualElement.projection) === null || _a === void 0 ? void 0 : _a.willUpdate();
        }
        return null;
    };
    MeasureLayoutWithContext.prototype.componentDidUpdate = function () {
        var _a;
        (_a = this.props.visualElement.projection) === null || _a === void 0 ? void 0 : _a.root.didUpdate();
    };
    MeasureLayoutWithContext.prototype.componentWillUnmount = function () {
        var group = this.context.group;
        if (group)
            group.remove(this.props.visualElement.projection);
    };
    MeasureLayoutWithContext.prototype.safeToRemove = function () {
        var _a, _b;
        (_b = (_a = this.props).safeToRemove) === null || _b === void 0 ? void 0 : _b.call(_a);
    };
    MeasureLayoutWithContext.prototype.render = function () {
        var _this = this;
        var _a;
        var _b = this.props, visualElement = _b.visualElement, layoutId = _b.layoutId, layout = _b.layout, drag = _b.drag;
        if (!visualElement.projection) {
            // TODO do this on construction
            var parent_1 = (_a = visualElement.parent) === null || _a === void 0 ? void 0 : _a.projection;
            visualElement.projection = new HTMLProjectionNode(this.props.projectionId, visualElement.getLatestValues(), parent_1);
            visualElement.projection.setOptions({
                layoutId: layoutId,
                layout: layout,
                alwaysMeasureLayout: !!drag,
                visualElement: visualElement,
                onProjectionUpdate: function () { return visualElement.scheduleRender(); },
                onExitComplete: function () { return _this.safeToRemove(); },
                animationType: typeof layout === "string" ? layout : "both",
            });
        }
        return null;
    };
    return MeasureLayoutWithContext;
}(React__default.Component));
function MeasureLayout(props) {
    var _a = __read(usePresence(), 2), isPresent = _a[0], safeToRemove = _a[1];
    return (React__default.createElement(MeasureLayoutWithContext, __assign({}, props, { layoutGroup: useContext(LayoutGroupContext), isPresent: isPresent, safeToRemove: safeToRemove })));
}
var defaultScaleCorrectors = {
    borderRadius: __assign(__assign({}, correctBorderRadius), { applyTo: [
            "borderTopLeftRadius",
            "borderTopRightRadius",
            "borderBottomLeftRadius",
            "borderBottomRightRadius",
        ] }),
    borderTopLeftRadius: correctBorderRadius,
    borderTopRightRadius: correctBorderRadius,
    borderBottomLeftRadius: correctBorderRadius,
    borderBottomRightRadius: correctBorderRadius,
    boxShadow: correctBoxShadow,
};

export { MeasureLayout };
